      program hist
c  *************************************************************
c  *
c  *  histogrm generates histograms and statistics on
c  *       all 8 and 16 bits/pixel REMAPP files
c  *
c  *  DN values may range from 0 - 32,767  (all positive only)
c  *
c  *  4 byte integers range from 0 - 2,147,483,647
c  *     this program can handle over 99,999sl x 8192 pixel
c  *     and stay below the 2 billion max (819,191,808)
c  *
c  *  K. Eric Livo  May, 1992
c  *     (after Don Sawatzky's original)
c  *
c  *  link with diskio and associated subroutines, and unpack, getnums
c  *
c  *  variables:
c  *
c  *  histogrm(i)   # of pixels at this dn class frequency
c  *  mindn     min dn (gray level) in sample (user selected interval)
c  *  maxdn     max dn (gray level) in sample (user selected interval)
c  *  lowestdn  min dn in image
c  *  highstdn  max dn in image
c  *  mean      average dn within sample interval  (X-bar sample population)
c  *  median    median dn within sample interval
c  *  mode      mode dn within sample interval (class interval with max count)
c  *  pixused   # of pixels used within sample  (mindn - maxdn)
c  *  total     # of pixels total in image (pixused <= total)
c  *  lowpix    # of pixels below sample interval (mindn)
c  *  highpix   # of pixels above sample interval (maxdn)
c  *  maxhstdn  max # of pixels in a class interval
c  *  maxi2     max dn for given bits/pixel  255 for 8bit, 32767 for 16bit
c  *
c  *  capital letters in varables below for people only
c  *          FORTRAN compilers are not case sensitive
c  *
c  *  sSQR     variance - sample population  (tested subset of population)
c  *                 sSQR = SS/(n-1)
c  *
c  *  SS        corrected sum of squares
c  *                 SS = sum from i=1 to n ((Xi - X-bar)squared)
c  *
c  *  s         standard deviation
c  *                 s = positive square root of s-sqr
c  *
c  *  oneprcnt  dn at one percent of sample interval population
c  *  twoprcnt  dn at two percent of sample interval population
c  *  nineight  dn at 98 % of sample interval population
c  *  ninenine  dn at 99 % of sample interval population
c  *
c  *************************************************************
 
CMSPC      integer*4 histogrm[FAR](0:32767)
      integer*4 histogrm[FAR](0:32767)
CBSD4      integer*4 histogrm(0:32767)
      integer*4 pixsum, i, sl, pix, dn
      integer*4 oneprcnt, twoprcnt, nineight, ninenine, cbitint
      integer*4 fcb(256), icumlen, maxi2, classint, classcnt, histsum
      integer*4 mindn, maxdn, lowpix, highpix, pixused, total, punit
      integer*4 lowestdn, highstdn, median, mode, maxhstdn
      integer*2 ibuf(8192)
      character cumgrph*51, cumgrphb*52, title*80, DEVICE*80
      real*8 sSQR, SS, s
      real*8 cumper, ocumper, mean
      logical *1 LEXIST
 
c  for subroutine getnums(comstr,inum)
 
      integer*4 inum(10)
      character*80 comstr
 
C.....OPEN LISTING FILE,  to consule, printer, or diskfile
 
      LEXIST = .TRUE.
 
      DO WHILE (LEXIST)
CMSPC         WRITE(6,*) 'Listing device (CON, PRN, OR diskfile)? '
         WRITE(6,*) 'Listing device (CON, PRN, OR diskfile)? '
CBSD4         WRITE(6,*) 'Listing device (CON, OR diskfile)? '
         READ(5,'(A80)') DEVICE
         IF ((DEVICE .NE. 'CON') .AND. (DEVICE .NE. 'con')) THEN
CMSPC            IF ((DEVICE .NE. 'PRN') .AND. (DEVICE .NE. 'prn')) THEN
            IF ((DEVICE .NE. 'PRN') .AND. (DEVICE .NE. 'prn')) THEN
               INQUIRE(FILE=DEVICE,EXIST=LEXIST)
CMSPC            ELSE
            ELSE
CMSPC               LEXIST = .FALSE.
               LEXIST = .FALSE.
CMSPC            ENDIF
            ENDIF
 
            IF (LEXIST) THEN
               WRITE(6,*) 'FILE EXISTS: ', DEVICE
               WRITE(6,*) 'REENTER NEW FILENAME'
            ELSE
               punit = 20
               OPEN(punit,FILE=DEVICE,STATUS='NEW')
            ENDIF
         ELSE
            LEXIST = .FALSE.
            punit = 6
         ENDIF
      ENDDO
 
c  log it  --  one time only
 
      call remlog('*****---  HIST  ---*****')
 
c  initialize  --  START MAIN LOOP on number of files to read
 
60    do 70 i=0, 32767
         histogrm(i) = 0
70    continue
      do 71 i=1, 256
         fcb(i) = 0
71    continue
      do 72 i=1, 8192
         ibuf(i) = 0
72    continue
 
      oneprcnt = -1
      twoprcnt = -1
      nineight = -1
      ninenine = -1
 
      cumgrph  = '***************************************************'
      cumgrphb = '+         +         +         +         +         + '
 
c  open input file
 
      fcb(1) = 0
 
      call diskio(0,9,ibuf,fcb)
      if(fcb(1).lt.0) call exit(3)
 
c  set maximum DN for data
 
      if(fcb(25).gt.8) then
         maxi2 = 32767
      else
         maxi2 = 255
      endif
 
 
c  enter title for output
 
      write(6,*) 'enter TITLE for output (80 charcters) or <return>'
      read(5,90) title
90    format(A80)
      write(6,*)
 
c  read min & max DN's to use, calc interval to test
 
      mindn = -1
      do while ((mindn.lt.0).or.(maxdn.gt.maxi2).or.(mindn.gt.maxdn))
         write(6,*) 'enter MINIMUM and MAXIMUM DNs to test:',
     &      '   <return> for max range'
         write(6,80) maxi2
80       format(1x,'  (DN permissible range: 0  to', i6,')')
         read(5,81) comstr
81       format(a)
 
         inum(1) = 2
         call getnums(comstr, inum)
         if(inum(1).gt. -2) then
            mindn = inum(1)
            if(mindn.le.0) mindn = 0
            maxdn = inum(2)
            if(maxdn.le.0) maxdn = maxi2
         endif
 
      enddo
 
c  read class interval for histogram
 
      cbitint = 1
      if(fcb(25).gt.8) cbitint = 10
 
      classint = -1
      do while ((classint.lt.1) .or. (classint.gt.maxi2))
         write(6,83) cbitint
83       format(1x,'enter CLASS INTERVAL:   <return> for interval of ',
     &      i2)
         write(6,84) maxi2
84       format(1x,'  CLASS INTERVAL permissible: 1  to',i6,')')
         read(5,85) comstr
85       format(a)
 
         inum(1) = 1
         call getnums(comstr, inum)
         if(inum(1).gt. -2) then
            classint = inum(1)
            if(classint.le.-1) classint = cbitint
         endif
 
      enddo
 
c  start read loop  (scanline max = fcb(3))
 
      total = 0
      pixused = 0
      lowpix = 0
      highpix = 0
      lowestdn = maxi2 + 1
      highstdn = -1
 
      do 100,sl=1,fcb(3)
 
c  read input file
 
         call diskio(9,9,ibuf,fcb)
         if(fcb(1).lt.0) goto 999

c  version 3 remapp - just: call unpack(ibuf,fcb)
         if (fcb(25).eq.8) call unpack(ibuf,fcb)
 
c  store stats (histogram, total sample pixels used, total pixels in image,
c               number of pixels out of sample range; low or high,
c               and lowest and highest DN found in image
 
         do 110, pix=1,fcb(2)
            if((ibuf(pix).ge.mindn).and.(ibuf(pix).le.maxdn)) then
               pixused = pixused + 1
            else
               if(ibuf(pix).lt.mindn) then
                  lowpix = lowpix + 1
               else
                  highpix = highpix + 1
               endif
            endif
            histogrm(ibuf(pix)) = histogrm(ibuf(pix)) + 1
            total = total + 1
            if(lowestdn.gt.ibuf(pix)) lowestdn = ibuf(pix)
            if(highstdn.lt.ibuf(pix)) highstdn = ibuf(pix)
110      continue
 
100   continue
 
c  set stats DN test interval
c     starting DN: use the higher of MINDN or LOWESTDN
c     ending DN:   use the lower of MAXDN or HIGHSTDN
c  (MINDN and MAXDN define the testing DN interval)
 
      if(mindn.gt.lowestdn) then
         statstrt = mindn
      else
         statstrt = lowestdn
      endif
      if(maxdn.lt.highstdn) then
         statend = maxdn
      else
         statend = highstdn
      endif
 
c  calc median DN (found at cumulative freq = 50%)
c     also start stats on mean DN, mode DN, mode DN count
 
      dn = statstrt - 1
      cumper = 0.
      maxhstdn = 0
      mean = 0.
      mode = 0
      pixsum = 0
      histsum = 0
      classcnt = 0
      do while (cumper.lt.50.0)
         ocumper = cumper
         dn = dn + 1
         classcnt = classcnt + 1
         histsum = histsum + histogrm(dn)
         if (classcnt.eq.classint) then
            if (histsum.gt.maxhstdn) then
               maxhstdn = histsum
               mode = dn
            endif
            histsum = 0
            classcnt = 0
         endif
         pixsum = pixsum + histogrm(dn)
         cumper = (dble(pixsum)/dble(pixused))*dble(100)
         mean = mean + (( dble(histogrm(dn)) / dble(pixused))*dble(dn))
         if((cumper .gt. 1.0) .and. (oneprcnt .eq. -1)) oneprcnt = dn
         if((cumper .gt. 2.0) .and. (twoprcnt .eq. -1)) twoprcnt = dn
      end do
 
      if((50.0 - ocumper) .lt. (cumper - 50.0)) then
         median = dn - 1
      else
         median = dn
      endif
 
c  finish mean, mode, and mode count
 
      i = dn + 1
      do 102 dn=i, statend
         classcnt = classcnt + 1
         histsum = histsum + histogrm(dn)
         if ((classcnt.eq.classint).or.(dn.eq.highstdn)) then
            if (histsum.gt.maxhstdn) then
               maxhstdn = histsum
               mode = dn
            endif
            histsum = 0
            classcnt = 0
         endif
         mean = mean + (( dble(histogrm(dn)) / dble(pixused))*dble(dn))
         pixsum = pixsum + histogrm(dn)
         cumper = (dble(pixsum)/dble(pixused))*dble(100)
         if((cumper .gt. 98.0) .and. (nineight .eq. -1)) nineight = dn
         if((cumper .gt. 99.0) .and. (ninenine .eq. -1)) ninenine = dn
102   continue
 
c  calc stats: sum of squares, variance, and std. deviation
 
      SS = 0.
      do 104 dn = statstrt, statend
         SS = SS + dble(histogrm(dn)) * ( (dble(dn)-mean)**2 )
104   continue
      sSQR = SS / (dble(pixused) - 1.)
      s = dsqrt(sSQR)
 
c  output DN, FREQ, and cum%
 
c  write to log file
 
      write(4,300) title
      write(4,305) mindn, maxdn
      write(4,310) lowestdn, highstdn
      write(4,320)  mean, median
      write(4,330) mode, maxhstdn
      write(4,340) pixused
      write(4,345) total
      write(4,350) lowpix, highpix
      write(4,360) sSQR, s
 
c  print to screen
 
      write(punit,300) title
      write(punit,305) mindn, maxdn
      write(punit,310) lowestdn, highstdn
      write(punit,320)  mean, median
      write(punit,330) mode, maxhstdn
      write(punit,340) pixused
      write(punit,345) total
      write(punit,350) lowpix, highpix
      write(punit,360) sSQR, s
      write(punit,370) oneprcnt, twoprcnt, nineight, ninenine
 
300   format(1x,a)
305   format(1x,'user selected: minimum sample DN ',i5,
     &          ';   maximum sample DN ',i5)
310   format(1x,'DISKIO subset: minimum image DN  ',i5,
     &          ';   maximum image DN  ',i5)
320   format(1x,'sample mean DN ',F7.1,';  sample median DN ',i5)
330   format(1x,'sample mode DN ',i5,';    occurs ',i10,'    times')
340   format(1x,'sample pixels used in histogram ',i10)
345   format(1x,'total pixels in DISKIO subset   ',i10)
350   format(1x,'pixels below sample interval',i10,
     &          ';  pixels above sample interval',i10)
360   format(1x,'sample variance ',F12.3,
     &          ';     sample standard deviation ',F9.3)
370   format(1x,'sample 1% ',i5,';    2% ',i5,';    98% ',i5,
     &          ';    99% ',i5,';')
 
      write(punit,*)
      write(punit,*)
      write(punit,220)
      write(punit,221)
      write(punit,222)
      write(punit,223)
      write(punit,*)
      write(punit,224)
 
220   format(1x,'  DN   frequency  cumulative         relative'
     & ' percent frequency')
221   format(1x,'(gray    (pixel    percent')
222   format(1x,'level)   counts)  frequency')
223   format(1x,T26,'0',T35,'20',T45,'40',T55,'60',T65,'80',T74,'100')
224   format(1x,T26,'+',T36,'+',T46,'+',T56,'+',T66,'+',T76,'+')
 
      pixsum = 0
      histsum = 0
      classcnt = 0
      do 200 dn=statstrt, statend
         pixsum = pixsum + histogrm(dn)
         classcnt = classcnt + 1
         histsum = histsum + histogrm(dn)
         if((classcnt .eq. classint) .or. (dn .eq. statend)) then
            icumlen = int((real(histsum)/real(maxhstdn))*50.) + 1
            cumper = (dble(pixsum) / dble(pixused)) * dble(100)
            write(punit,210) dn, histsum, cumper, cumgrph(1:icumlen)
     &      // cumgrphb(icumlen+1:52)
210         format(1x, I5, I11, F7.1, ' ', A)
            histsum = 0
            classcnt = 0
         endif
200   continue
 
      write(punit,225)
225   format(1x,T26,'+',T36,'+',T46,'+',T56,'+',T66,'+',T76,'+')
 
c  finish with this image
 
999   call diskio(6,9,ibuf,fcb)
 
c  test for more images to run histogram on & loop
 
      write(6,*) 'Process another file ?  Yes or No  :  <Y> default '
      read(5,230) comstr
230   format(A1)
      if((comstr .ne. 'N') .and. (comstr .ne. 'n')) goto 60
 
      stop
      end
 
 
 
 
 
      subroutine getnums(comstr,inum)
c  *************************************************************
c  *
c  *  subroutine generates integer*4's from character
c  *     command input string
c  *
c  *  only numeric charcters, commas, and spaces accepted;
c  *     all else generates inum(1) = -2 error
c  *
c  *  numeric characters assembled into integers (int*4)
c  *  unassigned integer fields set to -1    ( inum(i) = -1 )
c  *  commas or spaces delimit numbers
c  *
c  *  command string:
c  *     '   234  987,,12'  or '234    987,   ,   12'
c  *  generates numbers:
c  *     inum(1)=234, inum(2)=987, inum(3)=-1, inum(4)=12
c  *
c  *  character string length and inum array size
c  *     set by main program
c  *
c  *
c  *  USEAGE:
c  *
c  *  input command string into comstr
c  *
c  *  input number of integer arguments into inum(1)
c  *
c  *  output: inum(1) >= -1   no errors
c  *          inum(i) = -1 to positive numbers (i=1-# of args)
c  *
c  *  output: inum(1) = -2   error's  returned inum(i's) no good
c  *             either unpermitted characters in string
c  *             or more arguments present than requested by inum(1)
c  *
c  *************************************************************
 
      integer*4 inum(1), strlen, iarg, maxarg, progerr, i
      character*(*) comstr
      character*1 comchar
 
      strlen = len(comstr)
      maxarg = inum(1)
 
c  zero integer array (to max argument used)
 
      do 110 i=1, maxarg
         inum(i) = -1
110   continue
 
      progerr = 0
      iarg = 1
 
c  parse integer numbers from character string comstr
c     put them in the inum() array
 
      do 120 i=1, strlen
         comchar = comstr(i:i)
 
         if(comchar .ge. '0' .and. comchar .le. '9') then
            if(inum(iarg).eq.-1) inum(iarg) = 0
            if(iarg.le.maxarg) then
               inum(iarg) = (inum(iarg)*10) + (ichar(comchar)-48)
            else
               progerr = -2
            endif
         else
            if(comchar .eq. ',') then
               if(iarg.le.maxarg) then
                  iarg = iarg + 1
               else
                  progerr = -2
               endif
            else
               if(comchar .eq. ' ') then
                  if((iarg.le.maxarg).and.(inum(iarg).gt.-1))
     &            iarg=iarg+1
               else
                  progerr = -2
               endif
            endif
         endif
 
120   continue
      if(progerr.lt.0) inum(1) = -2
 
      return
      end
