      PROGRAM MASK
C  *************************************************************************
C  *   by: K. Eric Livo                                                    *
C  *                                                                       *
C  *   Masking routine - input: Background Image, Overlay Image,           *
C  *      Mask Image, DN mask interval, and if selected, DN mask value.    *
C  *                                                                       *
C  *   Masking: when the DN value of a pixel from the Mask Image falls     *
C  *      within the selected DN mask interval, the DN value from the same *
C  *      pixel on the Overlay Image is used in the Output Image.  When    *
C  *      the DN value of a pixel from the Mask Image falls outside the    *
C  *      selected DN mask interval, the DN value from the same pixel on   *
C  *      the Background Image is used in the Output Image.  If a DN       *
C  *      constant value is selected for either the Background or Overlay  * 
C  *      Image, this value is used instead of an Image.                   *
C  *                                                                       *
C  *   The Background Image and/or Overlay Image  (which ever is greater)  *
C  *      defines the output DN range  (8 or 16 bits per pixel).           *
C  *                                                                       *
C  *   LINK WITH DISKIO, PACK, UNPACK                                      *
C  *                                                                       *
C  *************************************************************************
C
      INTEGER*2 BACKGRND(8192),OVERLAY(8192),IMASK(8192)
      INTEGER*4 FCBBKGD(256),FCBOVLY(256),FCBMASK(256),FCB(256)
      INTEGER*4 MAXSL, MAXPIX, MAXDN, MDN
      INTEGER*2 I,LOWDN,HIDN
      CHARACTER*1 KEY, KEY2, KEY3
      DATA FCBBKGD,FCBOVLY,FCBMASK,FCB/1024*0/
      LINES=0
      CALL REMLOG('*****--- MASK ---*****')
      MAXPIX=8192
      MAXSL=100000000
      MAXDN=8
C
      WRITE(6,5)
5     FORMAT(1X,'do you want instructions (y/n): ',\)
      READ(5,'(A1)') KEY
      IF ((KEY.EQ.'Y').OR.(KEY.EQ.'y')) THEN
      WRITE(6,*) 'This program combines two images into one output.'
      WRITE(6,*) 'The combination is controlled by a mask image.'
      WRITE(6,*)
      WRITE(6,*) '     A DN range is user selected, which defines' 
      WRITE(6,*) 'how the mask image combines the two input images' 
      WRITE(6,*) '(background and overlay).'
      WRITE(6,*) '     Either (but not both) input image may be'
      WRITE(6,*) 'defined as a homogeneous single DN constant' 
      WRITE(6,*) 'internal array image (not an external disk file).'
      WRITE(6,*)
      WRITE(6,*) 'When a mask pixel is within the defined range,'
      WRITE(6,*) 'the output pixel is taken from the overlay image.'
      WRITE(6,*)
      WRITE(6,*) 'When the mask pixel is outside the defined range,'
      WRITE(6,*) 'the output pixel is taken from the background image.'
      WRITE(6,*)
      ENDIF
C
C  select whether 3 images or 2 images and a DN constant are used for masking
C
10    WRITE(6,*)
      WRITE(6,*) 'Choose - Mask Image used with: '
      WRITE(6,*) '         <1>  Background and Overlay Images'
      WRITE(6,*) '         <2>  Input Image (Background or Overlay)',
     & ' and DN constant'
      WRITE(6,*)
      WRITE(6,11)
11    FORMAT(1X,'Press Key 1 or 2: ',\)
      READ(5,'(A1)') KEY
      IF((KEY.NE.'1').AND.(KEY.NE.'2')) GOTO 10
C
C  select which image (Background or Overlay) the DN constant
C     is substituted for.
C
      KEY2 = '0'
      IF(KEY.EQ.'2') THEN
20       WRITE(6,*)
         WRITE(6,*) 'Choose: '
         WRITE(6,*) '         <1>  Background Image and DN constant'
         WRITE(6,*) '         <2>  Overlay Image and DN constant'
         WRITE(6,*)
         WRITE(6,21)
21       FORMAT(1X,'Press Key 1 or 2: ',\)
         READ(5,'(A1)') KEY2
         IF((KEY2.NE.'1').AND.(KEY2.NE.'2')) GOTO 20
      ENDIF
C
C  open background image
C
      IF(KEY2.NE.'2') THEN
         WRITE(6,*) 'Open Background Image'
         FCBBKGD(1)=0
         CALL DISKIO(0,10,BACKGRND,FCBBKGD)
         IF(FCBBKGD(1).LT.0) CALL EXIT(10)
         MAXPIX=FCBBKGD(2)
         MAXSL=FCBBKGD(3)
         MAXDN=FCBBKGD(25)
      ELSE
         WRITE(6,*)
         WRITE(6,31)
31       FORMAT(1X,'Enter DN for background value: ',\)
         READ(5,*) MDN
         IF(MDN.GT.255) MAXDN=16
         DO 35 I=1,MAXPIX
            BACKGRND(I)=MDN
35       CONTINUE
      ENDIF
C
C  open overlay (foreground) image
C
      IF(KEY2.NE.'1') THEN
         WRITE(6,*) 'Open Overlay Image'
         FCBOVLY(1)=0
         CALL DISKIO(0,11,OVERLAY,FCBOVLY)
         IF(FCBOVLY(1).LT.0) CALL EXIT(11)
         IF(MAXPIX.GT.FCBOVLY(2)) MAXPIX=FCBOVLY(2)
         IF(MAXSL.GT.FCBOVLY(3)) MAXSL=FCBOVLY(3)
         IF(MAXDN.LT.FCBOVLY(25)) MAXDN=FCBOVLY(25)
      ELSE
         WRITE(6,*)
         WRITE(6,41)
41       FORMAT(1X,'Enter DN for overlay value: ',\)
         READ(5,*) MDN
         IF(MDN.GT.255) MAXDN=16
         DO 50 I=1,MAXPIX
            OVERLAY(I)=MDN
50       CONTINUE
      ENDIF
C
C  open mask image
C
60    WRITE(6,*)
      WRITE(6,61)
61    FORMAT(1X,'Is Mask Image also the Background or',
     & ' Overlay Image? (Y/N): ',\)
      READ(5,'(A1)') KEY3
      IF((KEY3.NE.'Y').AND.(KEY3.NE.'y').AND.(KEY3.NE.'N')
     & .AND.(KEY3.NE.'n')) GOTO 60
      IF(((KEY3.EQ.'Y').OR.(KEY3.EQ.'y')).and.(KEY2.EQ.'0')) THEN
70       WRITE(6,*)
         WRITE(6,*) 'Choose:'
         WRITE(6,*) '        <1>  Background Image used as Mask Image'
         WRITE(6,*) '        <2>  Overlay Image used as Mask Image'
         WRITE(6,*)
         WRITE(6,71)
71       FORMAT(1X,'Press Key 1 or 2: ',\)
         READ(5,'(A1)') KEY3
         IF((KEY3.NE.'1').AND.(KEY3.NE.'2')) GOTO 70
      ELSE
         IF((KEY3.EQ.'Y').OR.(KEY3.EQ.'y')) THEN
            KEY3(1:1)=KEY2(1:1)
         ELSE
            KEY3='0'
            WRITE(6,*) 'Open Mask Image'
            FCBMASK(1)=0
            CALL DISKIO(0,12,IMASK,FCBMASK)
            IF(FCBMASK(1).LT.0) CALL EXIT(12)
            IF(MAXPIX.GT.FCBMASK(2)) MAXPIX=FCBMASK(2)
            IF(MAXSL.GT.FCBMASK(3)) MAXSL=FCBMASK(3)
         ENDIF
      ENDIF
C
C  open output image
C
      FCB(1)=MAXDN
      FCB(2)=MAXPIX
      IF(FCBBKGD(22).GT.1)THEN
         FCB(13)=1
      ELSE
         FCB(13)=FCBBKGD(13)
      ENDIF
      WRITE(6,*) 'Open Output Image'
      CALL DISKIO(0,13,BACKGRND,FCB)
      IF(FCB(1).LT.0) CALL EXIT(13)
C
C  input masking DN interval
C
      WRITE(6,*) 'Enter DN Mask Interval '
      READ(5,*) LOWDN,HIDN
      WRITE(4,100) LOWDN,HIDN
100   FORMAT(1X,'DN Mask Interval',2I6)
C
C  start output loop
C
200   IF(KEY2.NE.'2') THEN
         CALL DISKIO(9,10,BACKGRND,FCBBKGD)
         IF(FCBBKGD(1).LT.0) GOTO 999
         IF(FCBBKGD(25).EQ.8) CALL UNPACK(BACKGRND,FCBBKGD)
      ENDIF
      IF(KEY2.NE.'1') THEN
         CALL DISKIO(9,11,OVERLAY,FCBOVLY)
         IF(FCBOVLY(1).LT.0) GOTO 999
         IF(FCBOVLY(25).EQ.8) CALL UNPACK(OVERLAY,FCBOVLY)
      ENDIF
      IF(KEY3.EQ.'0') THEN
         CALL DISKIO(9,12,IMASK,FCBMASK)
         IF(FCBMASK(1).LT.0) GOTO 999
         IF(FCBMASK(25).EQ.8) CALL UNPACK(IMASK,FCBMASK)
      ENDIF
      IF(KEY3.EQ.'1') THEN
         DO 203 I=1,MAXPIX
            IMASK(I)=BACKGRND(I)
203      CONTINUE
      ENDIF
      IF(KEY3.EQ.'2') THEN
         DO 206 I=1,MAXPIX
            IMASK(I)=OVERLAY(I)
206      CONTINUE
      ENDIF
C
C  apply overlay image to background image when mask within DN interval
C
      DO 210 I=1,MAXPIX
      IF((IMASK(I).GE.LOWDN).AND.(IMASK(I).LE.HIDN)) THEN
         BACKGRND(I) = OVERLAY(I)
      ENDIF
210   CONTINUE
C
C  write output scanline
C
      IF (FCB(25).EQ.8) CALL PACK(BACKGRND,FCB)
      CALL DISKIO(10,13,BACKGRND,FCB)
      IF(FCB(1).LT.0) CALL EXIT(14)
      LINES=LINES+1
      IF(LINES.LT.MAXSL) GO TO 200
C
C  close files
C
999   CALL DISKIO(6,10,BACKGRND,FCBBKGD)
      CALL DISKIO(6,11,OVERLAY,FCBOVLY)
      CALL DISKIO(6,12,IMASK,FCBMASK)
      CALL DISKIO(6,13,BACKGRND,FCB)
      STOP
      END
      
